"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var helpers_1 = require("./helpers");
var socketMessage_1 = require("./socketMessage");
var connectionHelpers_1 = require("./connectionHelpers");
var DATA_PREFIX = 'm:';
var CONTROL_PREFIX = 'c:';
var StockingsConnection = /** @class */ (function () {
    function StockingsConnection(connection, tracker) {
        var _this = this;
        this._dataSubscribers = new Map();
        this._controlSubscribers = new Map();
        this._closeSubscribers = new Map();
        this._subscriptions = new Map();
        this._transactions = new Map();
        this._connection = connection;
        this._clientId = helpers_1.generateRandomId();
        this._tracker = tracker;
        this.dataObservable = connectionHelpers_1.makeSubscriberMapObservable(this._dataSubscribers);
        this.controlObservable = connectionHelpers_1.makeSubscriberMapObservable(this._controlSubscribers);
        this.closeObservable = connectionHelpers_1.makeSubscriberMapObservable(this._closeSubscribers);
        this._connection.on('message', function (msg) {
            var serialData = msg.utf8Data;
            connectionHelpers_1.sendMessageIfPrefixed(DATA_PREFIX, serialData, _this._dataSubscribers);
            connectionHelpers_1.sendMessageIfPrefixed(CONTROL_PREFIX, serialData, _this._controlSubscribers);
        });
        this._connection.on('close', function (code, desc) {
            connectionHelpers_1.sendData(_this._closeSubscribers, undefined);
            connectionHelpers_1.complete(_this._dataSubscribers);
            connectionHelpers_1.complete(_this._controlSubscribers);
            connectionHelpers_1.complete(_this._closeSubscribers);
        });
    }
    StockingsConnection.prototype.getId = function () {
        return this._clientId;
    };
    StockingsConnection.prototype.getClientAddress = function () {
        return this._connection.remoteAddress;
    };
    StockingsConnection.prototype.generateTransactionId = function () {
        return helpers_1.generateRandomId();
    };
    StockingsConnection.prototype.sendData = function (type, payload) {
        this._connection.send(DATA_PREFIX + socketMessage_1.serializeMessage(type, payload));
    };
    StockingsConnection.prototype.sendControl = function (type, payload) {
        this._connection.send(CONTROL_PREFIX + socketMessage_1.serializeMessage(type, payload));
    };
    StockingsConnection.prototype.listenData = function (type) {
        return this.dataObservable.filter(function (msg) { return msg.type === type; }).map(function (msg) { return msg.payload; });
    };
    StockingsConnection.prototype.listenControl = function (type) {
        return this.controlObservable.filter(function (msg) { return msg.type === type; }).map(function (msg) { return msg.payload; });
    };
    StockingsConnection.prototype.addSubscription = function (type, transactionId, mergeStrategy, upsertKey) {
        if (mergeStrategy === void 0) { mergeStrategy = 'replace'; }
        var transactionSubscriptions = [];
        if (this._transactions.has(transactionId)) {
            transactionSubscriptions = this._transactions.get(transactionId);
        }
        else {
            this._transactions.set(transactionId, transactionSubscriptions);
        }
        var value = 0;
        if (this._subscriptions.has(type)) {
            value = this._subscriptions.get(type);
        }
        if (transactionSubscriptions.find(function (sub) { return sub.type === type; })) {
            return value;
        }
        else {
            transactionSubscriptions.push({ type: type, mergeStrategy: mergeStrategy, upsertKey: upsertKey });
        }
        if (value === 0) {
            this._tracker.registerSubscription(this, type);
        }
        value++;
        this._subscriptions.set(type, value);
        return value;
    };
    StockingsConnection.prototype.removeSubscriptions = function (transactionId) {
        var _this = this;
        if (!this._transactions.has(transactionId)) {
            return;
        }
        var transactionSubscriptions = this._transactions.get(transactionId);
        transactionSubscriptions.forEach(function (sub) { return _this._removeSubscription(sub.type); });
        this._transactions.delete(transactionId);
    };
    StockingsConnection.prototype.getSubscriptionHeader = function (transactionId) {
        return JSON.stringify(this._buildTransaction(transactionId));
    };
    StockingsConnection.prototype.getSubscriptions = function (transactionId) {
        return this._transactions.get(transactionId).map(function (sub) { return sub.type; });
    };
    StockingsConnection.prototype.getAllSubscriptions = function () {
        return connectionHelpers_1.iterableToArray(this._subscriptions.keys());
    };
    StockingsConnection.prototype.addSubscriptionsFrom = function (other) {
        var _this = this;
        connectionHelpers_1.iterableForEach(other._transactions.keys(), function (transactionId) {
            other._transactions.get(transactionId).forEach(function (sub) { return _this.addSubscription(sub.type, transactionId, sub.mergeStrategy); });
        });
    };
    StockingsConnection.prototype.close = function () {
        this._connection.close();
        this._connection.clearCloseTimer();
    };
    StockingsConnection.prototype._removeSubscription = function (type) {
        if (!this._subscriptions.has(type)) {
            return 0;
        }
        var value = this._subscriptions.get(type);
        if (value > 0) {
            value--;
        }
        if (value === 0) {
            this._tracker.unregisterSubscription(this, type);
        }
        this._subscriptions.set(type, value);
        return value;
    };
    StockingsConnection.prototype._buildTransaction = function (transactionId) {
        return {
            transactionId: transactionId,
            subscriptions: this._transactions.get(transactionId)
        };
    };
    return StockingsConnection;
}());
exports.StockingsConnection = StockingsConnection;
