"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var websocket_1 = require("websocket");
var stockingsConnection_1 = require("./stockingsConnection");
var tokenHelpers_1 = require("./tokenHelpers");
var helpers_1 = require("./helpers");
var connectionHelpers_1 = require("./connectionHelpers");
var controlProtocol_1 = require("./controlProtocol");
var ONE_SECOND = 1000;
var ONE_MINUTE = 60 * ONE_SECOND;
var HALF_HOUR = 30 * ONE_MINUTE;
var STOCKINGS_PROTOCOL = 'stockings';
var StockingsServer = /** @class */ (function () {
    function StockingsServer(options) {
        var _this = this;
        this._connections = new Map();
        this._connectionsBySubscription = new Map();
        this._server = options.server;
        this._socketServer = new websocket_1.server({
            httpServer: this._server,
            autoAcceptConnections: false,
            keepalive: false
        });
        var algorithm = helpers_1.valueOrDefault(options.algorithm, 'HS256');
        if (tokenHelpers_1.isAlgorithmAsymmetric(algorithm) && (!options.privateKey || !options.publicKey)) {
            throw new Error('For asymmetric encryption algorithm ' + algorithm + ', both privateKey and publicKey properties are required.');
        }
        else if (!options.privateKey) {
            throw new Error('For encryption algorithm ' + algorithm + ', privateKey property is required.');
        }
        var privateKey = options.privateKey;
        var decryptionKey = tokenHelpers_1.isAlgorithmAsymmetric(algorithm) ? options.publicKey : privateKey;
        this._encoder = tokenHelpers_1.makeEncoder(algorithm, privateKey);
        this._decoder = tokenHelpers_1.makeDecoder(algorithm, decryptionKey);
        if (typeof options.requestFilter === 'function') {
            this._acceptRequest = options.requestFilter;
        }
        else {
            this._acceptRequest = function (req) { return true; };
        }
        var disposeMetaAfter = helpers_1.valueOrDefault(options.disposeMetaAfter, HALF_HOUR);
        this._socketServer.on('request', function (req) {
            if (!hasStockingsProtocol(req) || !_this._acceptRequest(req)) {
                req.reject();
                return;
            }
            var connection = new stockingsConnection_1.StockingsConnection(req.accept(STOCKINGS_PROTOCOL), _this);
            controlProtocol_1.applyProtocol({
                connection: connection,
                tokenEncoder: _this._encoder,
                getConnection: function (token) { return _this.getConnection(token, connection.getClientAddress()); }
            });
            _this._connections.set(connection.getId(), connection);
            var disposeMetaSubscription = connection.closeObservable.delay(disposeMetaAfter).subscribe(function () {
                _this._connections.delete(connection.getId());
                connection.getAllSubscriptions().forEach(function (type) {
                    _this.unregisterSubscription(connection, type);
                });
                disposeMetaSubscription.unsubscribe();
            });
        });
    }
    StockingsServer.prototype.getConnection = function (token, address) {
        var _this = this;
        return this._decoder(token).then(function (payload) {
            var connection = _this._connections.get(payload.clientId);
            if (address !== payload.address) {
                if (connection) {
                    connection.close();
                }
                return Promise.reject(new Error('IP address mismatch'));
            }
            return connection;
        });
    };
    StockingsServer.prototype.sendData = function (type, payload, cb) {
        var _this = this;
        if (!this._connectionsBySubscription.has(type)) {
            return;
        }
        setImmediate(function () {
            var subscribers = _this._connectionsBySubscription.get(type);
            connectionHelpers_1.iterableForEach(subscribers.values(), function (connection) {
                connection.sendData(type, payload);
            });
            if (typeof cb === 'function') {
                cb();
            }
        });
    };
    StockingsServer.prototype.registerSubscription = function (connection, type) {
        var connections;
        if (this._connectionsBySubscription.has(type)) {
            connections = this._connectionsBySubscription.get(type);
        }
        else {
            connections = new Map();
            this._connectionsBySubscription.set(type, connections);
        }
        connections.set(connection.getId(), connection);
    };
    StockingsServer.prototype.unregisterSubscription = function (connection, type) {
        if (!this._connectionsBySubscription.has(type)) {
            return;
        }
        var connections = this._connectionsBySubscription.get(type);
        connections.delete(connection.getId());
    };
    return StockingsServer;
}());
exports.StockingsServer = StockingsServer;
function hasStockingsProtocol(req) {
    return req.requestedProtocols.map(function (p) { return p.toLowerCase(); }).indexOf(STOCKINGS_PROTOCOL) > -1;
}
